%-------------------------------------------------------------------------%
% Copyright (c) 2018, Modenese L. and Erica Montefiori                    %
%                                                                         %
% Licensed under the Apache License, Version 2.0 (the "License");         %
% you may not use this file except in compliance with the License.        %
% You may obtain a copy of the License at                                 %
% http://www.apache.org/licenses/LICENSE-2.0.                             %
%                                                                         % 
% Unless required by applicable law or agreed to in writing, software     %
% distributed under the License is distributed on an "AS IS" BASIS,       %
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         %
% implied. See the License for the specific language governing            %
% permissions and limitations under the License.                          %
%                                                                         %
%    Author:   Luca Modenese, February 2016                               %
%    email:    l.modenese@sheffield.ac.uk                                 % 
% ----------------------------------------------------------------------- %
% This script is used to calculate the hip joint centre from stl geometries of
% the femoral head. The code performs a least square fitting.
% The scripts requires the user to specify a folder where the geometries
% are stored and assumes that it will contain two files named:
% - femur_head_l.stl
% - femur_head_r.stl
% that it will use for the fitting procedure.
% If generating a unilateral model, the user is supposed to comment the 
% code related to the contralateral side or errors will be thrown.
% ----------------------------------------------------------------------- %
% Last modified
% October 2016, Erica Montefiori
% Added printing of the output to Fitted_Geometries_side.txt file compatible with NMS Builder
% landmark clouds import tool. Now code browse Right and Left folders
% automatically.
% April 2018, Erica Montefiori
% Modified sphere plot.
% ----------------------------------------------------------------------- %

% initialize
clear;clc;
close all; fclose all; 

% add functions to path
addpath(genpath('./Fitting_functions'))
addpath(genpath('./ReadWrite_functions'))

% the user selects the directory
geometry_folder = uigetdir('./geometries','Select the folder containing the geometries');

%% set up log file
log_file = fullfile(geometry_folder,'fittingResults.txt');
% create file and clean it
fid = fopen(log_file,'w+');
fclose all;
% start logging
diary(log_file);
diary on

%% get geometry files
HJC_left_file = fullfile(geometry_folder,'femur_head_l.stl');
HJC_right_file = fullfile(geometry_folder,'femur_head_r.stl');

%log of where the data are
display('Processing data from folder: ')
display(char(geometry_folder));
display(' ');

% initialize figure
h = figure('Position', [1 41 1366 652]);

%% ====== fit sphere to femoral head surface (left) =====
% read the file
[v, f] = stlread(HJC_left_file);
[v, f]=patchslim(v, f);
[Center_LSE,Radius_LSE] = sphereFit(v);

display('======== HJC Left ==========')
display(['Sphere centre: ', num2str(Center_LSE)]);
display(['Sphere radius: ', num2str(Radius_LSE)]);

% plot sphere
subplot(1,2,1)
[X,Y,Z] = sphere(49);
s=surf(Center_LSE(1)+X*Radius_LSE,Center_LSE(2)+Y*Radius_LSE,Center_LSE(3)+Z*Radius_LSE,'FaceColor',[0 0 1]);
hold on
axis equal, grid on
plot3(v(:,1),v(:,2),v(:,3),'.r');
title({'HJC Left';['Centre: ', num2str(Center_LSE)]; ['Radius: ', num2str(Radius_LSE)]}, 'FontWeight', 'bold')


% update names
updPMat(:,1) = Center_LSE;
colheaders_upd{1} =  ('Femur_Head_Sphere_Centre');

%% save figures 
set(h, 'PaperPositionMode','auto')
saveas(h,fullfile(geometry_folder,'fittingResults.png'));
saveas(h,fullfile(geometry_folder,'fittingResults.fig'));
% close all



%% creating outputs and output files
% creating the structure with the snapped points
updLandmarkCloud = struct;
updLandmarkCloud.colheaders = colheaders_upd;
updLandmarkCloud.data = updPMat;


% write the updated landmark cloud in the same place where the original
% cloud is located
%[~, cloud_name, est] = fileparts(cloud_file);
%[~, stl_name, est] = fileparts(stl_file);
fitted_landmarks_file_name = ['Fitted_geometries_l.txt'];
writeNMSBuilderLandmarkCloud(updLandmarkCloud,fullfile(geometry_folder,fitted_landmarks_file_name));
%display(['NMSBuilder file ',fitted_landmarks_file_name,' written in folder: ', cloud_path])


%% ====== fit sphere to femoral head (right side) =====
% read the file
[v, f] = stlread(HJC_right_file);
[v, f]=patchslim(v, f);
[Center_LSE,Radius_LSE] = sphereFit(v);

display('======== HJC Right ==========')
display(['Sphere centre: ', num2str(Center_LSE)]);
display(['Sphere radius: ', num2str(Radius_LSE)]);

% plot sphere
subplot(1,2,2)
[X,Y,Z] = sphere(49);
s=surf(Center_LSE(1)+X*Radius_LSE,Center_LSE(2)+Y*Radius_LSE,Center_LSE(3)+Z*Radius_LSE,'FaceColor',[0 0 1]);
hold on
axis equal, grid on
plot3(v(:,1),v(:,2),v(:,3),'.r');
title({'HJC Right';['Centre: ', num2str(Center_LSE)]; ['Radius: ', num2str(Radius_LSE)]}, 'FontWeight', 'bold')

% update names
updPMat(:,1) = Center_LSE;
colheaders_upd{1} =  ('Femur_Head_Sphere_Centre');

%% save figures 
set(h, 'PaperPositionMode','auto')
saveas(h,fullfile(geometry_folder,'fittingResults.png'));
saveas(h,fullfile(geometry_folder,'fittingResults.fig'));
% close all

% inform the user
display(' ');
display('The plots and a log fils have been saved in geometry folder.');



%% creating outputs and output files
% creating the structure with the snapped points
updLandmarkCloud = struct;
updLandmarkCloud.colheaders = colheaders_upd;
updLandmarkCloud.data = updPMat;


% write the updated landmark cloud in the same place where the original
% clous is located
%[~, cloud_name, est] = fileparts(cloud_file);
%[~, stl_name, est] = fileparts(stl_file);
fitted_landmarks_file_name = ['Fitted_geometries_r.txt'];
writeNMSBuilderLandmarkCloud(updLandmarkCloud,fullfile(geometry_folder,fitted_landmarks_file_name));
%display(['NMSBuilder file ',fitted_landmarks_file_name,' written in folder: ', cloud_path])
diary off
